#!/usr/bin/env python
# IRCLogSearch.py: A simple gtk application for searching irc logs

## Copyright (C) 2009  Randal Barlow
##
## This file is part of IRCLogSearcher
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/


import sys
import gtk
from threading import Thread

from ui.logview import get_selected, append_results
from parsers.axeirc import search_file, get_file_area, filter_results
from ui.morelinesdialog import MorelinesDialog
from ui.logsearchui import UiBuilder


def thread_decorator(f):
    """"A Decorator to thread file search functions"""
    def wrapper(*args, **kwargs):
        t = Thread(target=f, args=args)
        t.start()
    return wrapper


class LogSearcherApp(object):
    """An instance which handles the LogSearcher application"""

    def __init__(self, filename = None):
        # Build the ui and obtain widgets
        self.Builder = UiBuilder(sys.path[0] + "/ui/interface.glade", self)
        self.widgets = self.Builder.widgets
        self.widgets.window.show_all()
        if filename:
            self.widgets.filechooserbutton.set_filename(filename)

    @thread_decorator
    def on_find(self, widget, *args, **kwargs):
        """Called when the find button is clicked or entry widget is activated"""
        self.widgets.resultstore.clear() # Clear the treeview
        filenames = self.widgets.filechooserbutton.get_filenames()
        if len(filenames) == 1:
            self.on_singlefile_find(widget, filenames[0], *args, **kwargs)
        elif len(filenames) > 1:
            self.on_multifile_find(widget, filenames, *args, **kwargs)
        else:
            print "No file selected"
        gtk.gdk.threads_enter()
        self.update_results_label(len(self.widgets.resultstore))
        gtk.gdk.threads_leave()

    def on_multifile_find(self, widget, filenames, *args, **kwargs):
        """A placeholder method for when I implement multiple files"""
        for filename in filenames:
            print filename

    def on_singlefile_find(self, widget, filename, *args, **kwargs):
        """Called when the find button is clicked or entry widget is activated"""
        if self.widgets.findbox.get_text() != "" or self.widgets.user_cb.get_active():
            results = search_file(filename, self.widgets.findbox.get_text(), self.widgets.case_cb.get_active(), self.widgets.regex_cb.get_active())
            if self.widgets.user_cb.get_active(): # check if the program should filter further by user
                results = filter_results(results, 1, self.widgets.user_entry.get_text(), self.widgets.case_cb.get_active(), False)
            gtk.gdk.threads_enter() # enter GUI thread
            append_results(self.widgets.resultstore, results, threaded=True) # put those results in a tree
            gtk.gdk.threads_leave() # leave GUI thread

    def on_copy_button_clicked(self, widget, *args, **kwargs):
        """Copies the selected item and places it into the GtkClipBoard"""
        resultitem = get_selected(self.widgets.resultsview, self.widgets.resultstore.get_n_columns())
        if resultitem:
            gtk.clipboard_get().set_text(resultitem[-1], -1)

    def update_results_label(self, result_count):
        """Updates the results label"""
        self.widgets.results_label.set_text("Results: %s" %result_count)

    def on_morelines_button_clicked(self, widget, *args, **kwargs):
        """Open a dialog that shows the lines surrounding the selected line"""
        resultitem = get_selected(self.widgets.resultsview, self.widgets.resultstore.get_n_columns())
        if resultitem:
            MorelinesDialog(get_file_area(self.widgets.filechooserbutton.get_filename(),
                                          resultitem[-1], self.widgets.morelines_spinbutton.get_value_as_int()))
