#!/usr/bin/python env
# sraper/axeirc.py: A module to strip data from irc logs

## Copyright (C) 2009  Randal Barlow
##
## This file is part of IRCLogSearcher
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/

import os
import re
import gobject

from logformats import XChatMessage as Message


def get_file_area(filename, line, nrange):
    logfile = file(filename)
    loglist = logfile.readlines()
    logfile.close()
    n = loglist.index(line + "\n")
    s = []
    for i in range(n-nrange,n+nrange+1):
        s.append(loglist[i])
    return "".join(str(e) for e in s)


def Search(line, query, checkcase):
    return (query in line) if checkcase else (query.lower() in line.lower())

def reSearch(line, query, checkcase):
    return re.search(query, line) if checkcase else re.search(query.lower(), line.lower())

def is_message(line):
    return (len(line) > 16 and line[0] != "*" and line[16] != "*" and line[16] != ">")


def search_file(filename, query, checkcase = False, regex = False):
    """
    A function which parses a log file quickly using a generator and finds matching lines using either
    RegEx or plain text.

    Returns a list of Message objects
    """
    logfile = file(filename)
    return (Message(line[:-1], filename) for line in logfile
            if is_message(line) if (reSearch(line, query, checkcase) if regex else Search(line, query, checkcase)))

def get_files(filenames, *args, **kwargs):
    """Returns a generator of generators!"""
    return (search_file(filename, *args, **kwargs) for filename in filenames)

def filter_results(results, column, query, checkcase, regex):
    """Filters the results by a user name and returns matching results"""
    return (x for x in results if (reSearch(x.User, query, checkcase) if regex else Search(x.User, query, checkcase)))
