#!/usr/bin/env python
# logsearch.py: Hold UI code to keep IRCLogSearcher.py clean

## Copyright (C) 2009  Randal Barlow
##
## This file is part of IRCLogSearcher
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/

import gtk


class WidgetHolder(object):
    def __init__(self, widget_names, builder):
        self.get_widgets(widget_names, builder)
        
    def get_widgets(self, widget_names, builder):
        """
        Iterating over a group of either list of string, or string a string and assigning it to a keyword
        
        widgets = [["window"_name_in_class", "window_name_ingladefile"], window_name_in_both]
        """
        # Now we check to see if the widget's name in the xml is the same as it is nammed in the glade file(list of strings vs a string)
        for w in widget_names:
            self.__dict__[w[0] if type(w) is list else w] = builder.get_object(w[1] if type(w) is list else w)

class UiBuilder(gtk.Builder):
    """
    A subclass of GtkBuilder with some helper methods added for ease of use
    """
    get_widget = gtk.Builder.get_object

    def __init__(self, glade_filename, app):
        super(UiBuilder, self).__init__()
        self.add_from_file(glade_filename)
        widget_names = ["window","findbox","resultsview","resultstore",
                        "user_entry","results_label","case_cb","regex_cb",
                        "user_cb","filechooserbutton","morelines_spinbutton"]
        self.widgets = WidgetHolder(widget_names, self)
        self.connect_signals({"gtk_main_quit": gtk.main_quit, 
                              "on_find-button_clicked" : app.on_find, 
                              "on_find-action_activate" : app.on_find,
                              "on_copy-button_clicked" : app.on_copy_button_clicked,
                              "on_morelines_button_clicked" : app.on_morelines_button_clicked})